#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include "BuilderWindow.h"
#include "BuilderMethods/panelIO.h"
#include "pmwcw_writer.h"
#include "BuilderMethods/rcsio.h"
#include "codewriter.h"
#include "Precognition3D.h"

int pmwcw_WriteMain( StringList *windows );

int pmwcw_WriteWindowHeader( BuilderWindow *panel );
int pmwcw_WriteWindowCode( BuilderWindow *panel );

int pmwcw_WriteWindow( char *windowname );

void WriteTheCode( FILE *file, BuilderWindow *panel, int section );

char *
simpleName( char *filename )
{
   int n, i;
   char *simplename;

   n = strlen(filename);
   for( i = n -1; i; i-- )
   {
      if( filename[i] == '/' || filename[i] == ':')
      {
         simplename = &filename[i+1];
         break;
      }
   }
   return simplename;
}

int
pmwcw_WriteCode( char *dir, StringList *windows )
{
   int i, status;

   status = TRUE;

   if( !pmwcw_WriteMain( windows ) ) return FALSE;

   for( i = 0; i < windows->nEntries; i++ )
   {
      if( !pmwcw_WriteWindow( windows->Entries[i] ) ) return FALSE;
   }
   return TRUE;
}

int
ReadWindowFromFile( BuilderWindow *panel, char *filename )
{
   int status;
   FILE *iff_file;

   status = FALSE;

   BuilderWindow_Init( panel, 0,12, 640, 200, StandardPens(), NULL, NULL, NULL );
   if( iff_file = fopen( filename,"r" ) )
   {
      status = ReadPrecogFile( iff_file, panel );
      fclose( iff_file );
   }
   else
      fprintf( stderr, "Couldn't open file \"%s\"\n", filename );

   if( status == 0 )
      CleanUp( (PObject *)&panel->pw );

   return status;
}

int
pmwcw_WriteWindow( char *filename )
{
   int status;
   BuilderWindow panel;

   status = FALSE;

   if( ReadWindowFromFile( &panel, filename ) )
   {
      if( pmwcw_WriteWindowHeader( &panel ) )
      {
         if( pmwcw_WriteWindowCode( &panel ) )
            status = TRUE;
      }
      CleanUp( (struct PObject *)&panel.pw );
   }

   return status;
}

#define FOR_ALL_WINDOWS for( i = 0; i < n; i++ ) { \
if( ReadWindowFromFile( &panel, windows->Entries[i] ) ) { \
windowname = PObjectName( (PObject *)&panel );

#define END_LOOP  CleanUp( (struct PObject *)&panel ); } }

int
pmwcw_WriteMain( StringList *windows )
{
   int i, n;
   RCSFILE *rcsfile;
   FILE    *file;
   char *windowname;
   BuilderWindow panel;

   if( ( rcsfile = rcs_create( "main.c" ) ) == NULL )
   {
      fprintf( stderr, "Error: Couldn't create \"main.c\"\n");
      return 0;
   }

   file = rcsfile->file;
   n = windows->nEntries;

   fprintf( file, "/* =====================================================\n" );
   fprintf( file, "**   Created by the Precognition Interface Builder\n" );
   fprintf( file, "** =====================================================\n" );
   fprintf( file, "**\n" );
   fprintf( file, "**  Link this file with 'Precognition.lib'\n" );
   fprintf( file, "**\n" );
   fprintf( file, "*/\n" );
   fprintf( file, "#include <stdlib.h>\n" );
   fprintf( file, "#include <stdio.h>\n" );
   fprintf( file, "#include \"Precognition.h\"\n" );
   fprintf( file, "#include \"Precognition3D.h\"\n" );
   fprintf( file, "#include \"Intuition_Utils.h\"\n" );
   fprintf( file, "#include <intuition/screens.h>\n" );
   fprintf( file, "#include <clib/exec_protos.h>\n" );
   fprintf( file, "#include <clib/alib_protos.h>\n" );
   fprintf( file, "#include <clib/intuition_protos.h>\n" );
   fprintf( file, "#include <clib/graphics_protos.h>\n" );
   fprintf( file, "#ifdef __SASC\n" );
   fprintf( file, "#include <proto/exec.h>\n" );
   fprintf( file, "#include <proto/intuition.h>\n" );
   fprintf( file, "#include <proto/graphics.h>\n" );
   fprintf( file, "#endif \n" );

   fprintf( file, "/* Window files */\n" );
   FOR_ALL_WINDOWS
      fprintf( file, "#include \"%s.h\"\n", windowname );
   END_LOOP

   fprintf( file, "\n" );
   fprintf( file, "/* declare lib structures */\n" );
   fprintf( file, "struct IntuitionBase *IntuitionBase = NULL;\n" );
   fprintf( file, "struct GfxBase       *GfxBase       = NULL;\n" );
   fprintf( file, "\n" );
   fprintf( file, "#define HAS_CONSOLE ( argc > 0 )\n" );
   fprintf( file, "/*\n" );
   fprintf( file, "** If 'argc' is greater than 0, than the program was started from\n" );
   fprintf( file, "** the command line, and therefore has a console that one can\n" );
   fprintf( file, "** 'fprintf' to.\n" );
   fprintf( file, "**\n" );
   fprintf( file, "** If 'argc' == 0, this was started from Workbench.  It's not\n" );
   fprintf( file, "** safe to assume there's a console attached to programs started\n" );
   fprintf( file, "** from Workbench.\n" );
   fprintf( file, "*/\n" );
   fprintf( file, "\n" );
   fprintf( file, "struct Screen *screen = NULL;\n" );
   fprintf( file, "BOOL Done = FALSE;\n" );
   fprintf( file, "/*\n" );
   fprintf( file, "** The event loop is terminated when 'Done' is true.\n" );
   fprintf( file, "*/\n" );
   fprintf( file, "\n" );
   fprintf( file, "struct MsgPort *shared_port; /* <== Intuition UserPort for all windows. */\n" );
   fprintf( file, "\n" );
   fprintf( file, "#ifdef ANSI_HEADERS\n");
   fprintf( file, "int  InitWindows( struct Screen*, struct MsgPort* );\n" );
   fprintf( file, "void EventLoop( struct MsgPort* );\n" );
   fprintf( file, "void CleanUpWindows( void );\n" );
   fprintf( file, "void GracefulExit( int );\n" );
   fprintf( file, "#else\n");
   fprintf( file, "int  InitWindows    ();\n" );
   fprintf( file, "void EventLoop      ();\n" );
   fprintf( file, "void CleanUpWindows ();\n" );
   fprintf( file, "void GracefulExit   ();\n" );
   fprintf( file, "#endif\n" );
   fprintf( file, "\n" );
   fprintf( file, "\n" );
   fprintf( file, "void main ( argc, argv )\n" );
   fprintf( file, "   int argc;\n" );
   fprintf( file, "   char **argv;\n" );
   fprintf( file, "{\n" );
   fprintf( file, "  /* open intuition library */\n" );
   fprintf( file, "   IntuitionBase = (struct IntuitionBase *)\n" );
   fprintf( file, "      OpenLibrary ( \"intuition.library\", 0 );\n" );
   fprintf( file, "   if( IntuitionBase == NULL )\n" );
   fprintf( file, "   {\n" );
   fprintf( file, "      if( HAS_CONSOLE ) \n" );
   fprintf( file, "         fprintf( stderr, \"Error: couldn't open intuition.library\\n\");\n" );
   fprintf( file, "      GracefulExit( 10 );\n" );
   fprintf( file, "   }\n" );
   fprintf( file, "\n" );
   fprintf( file, "   GfxBase = (struct GfxBase *)\n" );
   fprintf( file, "      OpenLibrary( \"graphics.library\", 0 );\n" );
   fprintf( file, "   if( GfxBase == NULL )\n" );
   fprintf( file, "   {\n" );
   fprintf( file, "      if( HAS_CONSOLE )\n" );
   fprintf( file, "         fprintf( stderr, \"Error: couldn't open graphics.library\\n\");\n" );
   fprintf( file, "      GracefulExit( 20 );\n" );
   fprintf( file, "   }\n" );
   fprintf( file, "\n" );
   fprintf( file, "   if( ( shared_port = CreatePort( NULL, 0 ) ) == NULL )\n" );
   fprintf( file, "   {\n" );
   fprintf( file, "      if( HAS_CONSOLE )\n" );
   fprintf( file, "         fprintf( stderr, \"couldn't create a Message port.\\n\");\n" );
   fprintf( file, "      GracefulExit( 30 );\n" );
   fprintf( file, "   }\n" );
   fprintf( file, "\n" );
   fprintf( file, "   /* Open a screen, if desired. */\n" );
   fprintf( file, "\n" );
   fprintf( file, "   if( InitWindows( screen, shared_port )  )\n" );
   fprintf( file, "   {\n" );
   fprintf( file, "      EventLoop( shared_port );\n" );
   fprintf( file, "   }\n" );
   fprintf( file, "   CleanUpWindows();\n" );
   fprintf( file, "\n" );
   fprintf( file, "   GracefulExit( 0 );\n" );
   fprintf( file, "}\n" );
   fprintf( file, "\n" );
   fprintf( file, "\n" );
   fprintf( file, "void GracefulExit( int status )\n" );
   fprintf( file, "{   \n" );
   fprintf( file, "   if( shared_port )   \n" );
   fprintf( file, "   {\n" );
   fprintf( file, "      DeletePort( shared_port );\n" );
   fprintf( file, "      shared_port = NULL;\n" );
   fprintf( file, "   }\n" );
   fprintf( file, "   if( GfxBase )         \n" );
   fprintf( file, "   {\n" );
   fprintf( file, "      CloseLibrary( (struct Library *)GfxBase );\n" );
   fprintf( file, "      GfxBase = NULL;\n" );
   fprintf( file, "   }\n" );
   fprintf( file, "   if( IntuitionBase )   \n" );
   fprintf( file, "   {\n" );
   fprintf( file, "      CloseLibrary( (struct Library *)IntuitionBase );\n" );
   fprintf( file, "      IntuitionBase = NULL;\n" );
   fprintf( file, "   }\n" );
   fprintf( file, "   exit( status );\n" );
   fprintf( file, "}\n" );
   fprintf( file, "\n" );
   fprintf( file, "\n" );
   fprintf( file, "int\n" );
   fprintf( file, "InitWindows( screen, userport )\n" );
   fprintf( file, "   struct Screen *screen;\n" );
   fprintf( file, "   struct MsgPort *userport;\n" );
   fprintf( file, "{\n" );
   fprintf( file, "   /* Initialize the windows ... */\n" );
   FOR_ALL_WINDOWS
      fprintf( file, "\n" );
      fprintf( file, "   %s_Init( &%s, screen );\n", windowname, windowname );
      fprintf( file, "   %s.w.SharedUserPort = userport;\n", windowname );
   END_LOOP
   fprintf( file, "\n" );
   fprintf( file, "   /* Open the windows */\n" );
   FOR_ALL_WINDOWS
      if( !( windows->Qualifiers[i] & ENTRY_SPECIAL ) )
         fprintf( file, "   if( pcgOpenWindow( &%s.w ) == NULL ) return FALSE;\n",
            windowname );
   END_LOOP
   fprintf( file, "\n" );
   fprintf( file, "   return TRUE;\n" );
   fprintf( file, "}\n" );
   fprintf( file, "\n" );
   fprintf( file, "void \n" );
   fprintf( file, "EventLoop( userport )\n" );
   fprintf( file, "   struct MsgPort *userport;\n" );
   fprintf( file, "{\n" );
   fprintf( file, "   struct IntuiMessage *imsg, event;   \n" );
   fprintf( file, "     \n" );
   fprintf( file, "   /*====== Event Loop ================*/\n" );
   fprintf( file, "   while( ! Done )\n" );
   fprintf( file, "   {\n" );
   fprintf( file, "      imsg = WaitForMessage( userport );\n" );
   fprintf( file, "      event = *imsg;\n" );
   fprintf( file, "      ReplyMsg( (struct Message *)imsg );\n" );
   fprintf( file, "      \n" );
   FOR_ALL_WINDOWS
      fprintf( file, "      %s_Respond( &%s, &event );\n", windowname, windowname );
   END_LOOP
   fprintf( file, "   }\n" );
   fprintf( file, "}\n" );
   fprintf( file, "\n" );
   fprintf( file, "void\n" );
   fprintf( file, "CleanUpWindows()\n" );
   fprintf( file, "{\n" );
   fprintf( file, "   /* Note: It's ok to close a Precognition window which is already closed. */\n\n");
   FOR_ALL_WINDOWS
      fprintf( file, "   pcgCloseWindow( &%s.w );\n", windowname );
      fprintf( file, "   %s_CleanUp( &%s );\n", windowname, windowname );
      fprintf( file, "\n" );
   END_LOOP

   fprintf( file, "}\n" );
   fprintf( file, "\n" );
   fprintf( file, "\n" );

   rcs_close( rcsfile );
   return TRUE;
}

char namebuf[80];

int pmwcw_WriteWindowHeader( BuilderWindow *panel )
{
   RCSFILE *rcsfile;
   FILE    *file;
   int      i, len, section;
   char    *windowname;

   windowname = PObjectName( (PObject *)panel );
   sprintf( &namebuf[0], "%s.h", windowname );

   if( ( rcsfile = rcs_create( &namebuf[0] ) ) == NULL )
   {
      fprintf( stderr, "Error: Couldn't create \"\"\n", &namebuf[0] );
      return 0;
   }

   file = rcsfile->file;

   sprintf( &namebuf[0], "%s_H", windowname );
   len = strlen(windowname);
   for( i = 0; i < len; i++ )
      namebuf[i] = toupper(namebuf[i]);

   fprintf( file, "#ifndef %s\n", &namebuf[0] );
   fprintf( file, "#define %s\n", &namebuf[0] );
   fprintf( file, "\n" );
   fprintf( file, "/* =====================================================\n" );
   fprintf( file, "**   Created by the Precognition Interface Builder\n" );
   fprintf( file, "** =====================================================\n" );
   fprintf( file, "*/\n" );
   fprintf( file, "#include \"Precognition.h\"\n" );
   fprintf( file, "#include \"Precognition3D.h\"\n" );
   fprintf( file, "\n" );

/* Write the structure and the prototypes for the window here. */
   for( section = CODE_BEGIN_WSTRUCT; section <= CODE_END_WSTRUCT; section++ )
   {
      WriteTheCode( file, panel, section );
   }

   fprintf( file, "\n" );
   sprintf( &namebuf[0], "%s_C", windowname );
   len = strlen( windowname );
   for( i = 0; i < len; i++ )
      namebuf[i] = toupper(namebuf[i]);

   fprintf( file, "#ifdef %s\n", &namebuf[0] );
   fprintf( file, "   struct %s %s;\n", windowname, windowname );
   fprintf( file, "#else\n" );
   fprintf( file, "   extern struct %s %s;\n", windowname, windowname );
   fprintf( file, "#endif\n" );
   fprintf( file, "\n" );

   WriteTheCode( file, panel, CODE_PROTOS );

   fprintf( file, "#endif\n" );

   rcs_close( rcsfile );
   return TRUE;
}

int pmwcw_WriteWindowCode( BuilderWindow *panel )
{
   RCSFILE *rcsfile;
   FILE    *file;
   char    *windowname;
   int i, len, section;

   windowname = PObjectName( (PObject *)panel );


   sprintf( &namebuf[0], "%s.c", windowname );

   if( ( rcsfile = rcs_create( &namebuf[0] ) ) == NULL )
   {
      fprintf( stderr, "Error: Couldn't create \"\"\n", &namebuf[0] );
      return 0;
   }

   file = rcsfile->file;

   fprintf( file, "/* =====================================================\n" );
   fprintf( file, "**   Created by the Precognition Interface Builder\n" );
   fprintf( file, "** =====================================================\n" );
   fprintf( file, "*/\n" );
   sprintf( &namebuf[0], "%s_C", windowname );
   len = strlen( windowname );
   for( i = 0; i < len; i++ )
      namebuf[i] = toupper(namebuf[i]);

   fprintf( file, "#define %s\n", &namebuf[0] );
   fprintf( file, "\n" );
   fprintf( file, "#include \"%s.h\"\n", windowname );
   fprintf( file, "\n" );
   fprintf( file, "#include \"Intuition_Utils.h\"\n" );
   fprintf( file, "#include <intuition/screens.h>\n" );
   fprintf( file, "#include <clib/exec_protos.h>\n" );
   fprintf( file, "#include <clib/intuition_protos.h>\n" );
   fprintf( file, "#include <clib/graphics_protos.h>\n" );
   fprintf( file, "#ifdef __SASC\n" );
   fprintf( file, "#include <proto/exec.h>\n" );
   fprintf( file, "#include <proto/intuition.h>\n" );
   fprintf( file, "#include <proto/graphics.h>\n" );
   fprintf( file, "#endif\n" );
   fprintf( file, "\n" );
   fprintf( file, "extern BOOL Done; /* Setting this to TRUE terminates the EventLoop. */\n" );
   fprintf( file, "\n" );
   WriteTheCode( file, panel, CODE_GLOBALS );
   fprintf( file, "\n" );

   for( section = CODE_CTOR_FUNC; section <= CODE_RESPOND_END; section++ )
   {
      WriteTheCode( file, panel, section );
   }

   rcs_close( rcsfile );
   return TRUE;
}

